@@ -13,8 +13,8 @@ if sys.platform == "win32":
     import colorama
     colorama.init(convert=True)
 
-from terminedia.input import keyboard, inkey, pause, KeyCodes, getch
-from terminedia.utils import Color, Rect, V2, Gradient
+from terminedia.input import keyboard, mouse, inkey, pause, KeyCodes, getch, input as sinput
+from terminedia.utils import Color, Rect, V2, Gradient, ColorGradient
 from terminedia.sprites import Sprite
 from terminedia.terminal import ScreenCommands, JournalingScreenCommands
 from terminedia.values import (
@@ -27,19 +27,30 @@ from terminedia.values import (
     NOP,
 )
 from terminedia.image import shape, ValueShape, ImageShape, PalettedShape, FullShape
-from terminedia.screen import Screen
 from terminedia.subpixels import BlockChars
 from terminedia.text import render
-from terminedia.transformers import Transformer
+from terminedia.text.style import Mark
+from terminedia.screen import Screen
+from terminedia.transformers import Transformer, TransformersContainer, GradientTransformer
+from terminedia.transformers.library import box_transformers as borders
+# Import otherwise unused modules, so that they are
+# always available after importing the main library:
+import terminedia.unicode
+from terminedia.events import EventTypes
+from terminedia.asynchronous import terminedia_main, ainput
+from terminedia.terminal import cls
 
-__version__ = "0.3.dev0"
+import terminedia.widgets
+
+
+__version__ = "0.4.dev0"
 __author__ = "João S. O. Bueno"
 
 
 print = ScreenCommands().print
 
 # These will be used for other backends than terminal rendering:
-context = _RootContext(default_fg="white", default_bg="black")
+context = _RootContext(default_fg="white", default_bg="black", fps=5)
 context.interactive_mode = sys.argv[0] == "" or sys.argv[0].endswith("ipython")
 context.fast_render = True
 
