@@ -1,7 +1,9 @@
 import inspect
+from collections.abc import Sequence
 
-from terminedia.subpixels import BlockChars
-from terminedia.values import CONTEXT_COLORS, EMPTY, TRANSPARENT
+from terminedia.image import RasterUndo
+from terminedia.subpixels import BlockChars, HalfChars
+from terminedia.values import CONTEXT_COLORS, EMPTY, TRANSPARENT, DEFAULT_BG, DEFAULT_FG, Directions
 from terminedia.utils import V2, Rect, contextkwords
 
 
@@ -18,7 +20,7 @@ class Drawing:
     That is - the typical usage for methods here will be ``screen.draw.line((0,0)-(50,20))``
     """
 
-    def __init__(self, set_fn, reset_fn, size_fn, context):
+    def __init__(self, set_fn, reset_fn, get_fn, size_fn, context):
         """Not intented to be instanced directly -
 
         Args:
@@ -38,6 +40,7 @@ class Drawing:
         and decide how to render it.
         """
         self.set = set_fn
+        self.get = get_fn
         self.reset = reset_fn
         self._size = size_fn
         self.context = context
@@ -47,6 +50,7 @@ class Drawing:
         return self._size()
 
     @contextkwords
+    @RasterUndo.undoable
     def line(self, pos1, pos2, erase=False):
         """Draws a straight line connecting both coordinates.
 
@@ -78,6 +82,7 @@ class Drawing:
             op(pos1.as_int)
 
     @contextkwords
+    @RasterUndo.undoable
     def rect(self, pos1, pos2=(), *, rel=(), erase=False):
         """Draws a rectangle
 
@@ -114,13 +119,55 @@ class Drawing:
 
     @contextkwords
     def fill(self):
-        """Fills the associated target with a solid color.
+        """Fills the associated target with current context values
         Args:
 
         The context attributes are used to fill the target area.
         """
         self.rect((0, 0), self.size, fill=True)
 
+    @contextkwords
+    @RasterUndo.undoable
+    def floodfill(self, pos, threshold=None):
+        """Fills the associated target with context values, starting at seed "pos".
+
+        Any different parameters for target pixels are considered boundaries
+        "threshold" may be passed a guard callable which will take each target
+        pixel, and should return "False" if it should be painted and "True"
+        if it is a boundary.
+        Args:
+            - pos (V2|Sequence[2]): initial position
+            - threshold (Optional[Callable[Pixel, Pixel, Pos]): callable - takes initial value at origin, and target pixel value.
+                    Should return 'True' for boundary pixels, False if pixel is to be painted.
+        The context attributes are used to fill the target area.
+        """
+        seed = self.get(pos)
+        if threshold is None:
+            threshold = lambda seed, target, pos: seed != target
+
+        rect = Rect(self.size)
+
+        fillable = set()
+        visited = set()
+
+        to_check = {pos,}
+
+        while to_check:
+            pos = to_check.pop()
+            if pos in visited:
+                continue
+            visited.add(pos)
+            if threshold(seed, self.get(pos), pos):
+                continue
+            fillable.add(pos)
+            for direction in Directions:
+                target = pos + direction
+                if target in rect:
+                    to_check.add(target)
+        for pos in fillable:
+            self.set(pos)
+
+
     def _link_prev(self, pos, i, limits, mask):
         if i < limits[0] - 1:
             for j in range(i, limits[0]):
@@ -132,6 +179,7 @@ class Drawing:
                 mask[j] = True
 
     @contextkwords
+    @RasterUndo.undoable
     def ellipse(self, pos1, pos2=(), *, rel=()):
         """Draws an ellipse
 
@@ -216,6 +264,7 @@ class Drawing:
             ox, oy = x, y
 
     @contextkwords
+    @RasterUndo.undoable
     def bezier(self, pos1, pos2, pos3, pos4, *extra):
         """Draws a bezier curve given the control points
 
@@ -250,6 +299,7 @@ class Drawing:
             self.bezier(pos4, extra[0], extra[1], extra[2], *extra[3:])
 
 
+    @RasterUndo.undoable
     def blit(self, pos, data, *, roi=None, color_map=None, erase=False):
         """Blits a blocky image in the associated screen at POS
 
@@ -361,7 +411,7 @@ class Drawing:
         self.context.background = self.context.background_stack.pop()
 
 
-class HighRes:
+class HighResBase:
     """ Provides a seamless mechanism to draw using unicode special characters as pixels.
 
     This is used as a base to have the method used for drawing using 1/4 block characters,
@@ -385,7 +435,7 @@ class HighRes:
 
         self.parent = parent
         self.draw = Drawing(
-            self.set_at, self.reset_at, self.get_size, self.parent.context
+            self.set_at, self.reset_at, self.get_at, self.get_size, self.parent.context
         )
         self.context = parent.context
 
@@ -394,10 +444,14 @@ class HighRes:
         w, h = self.parent.get_size()
         return V2(w * self.block_width, h * self.block_height)
 
+    size = property(get_size)
+    width = property(lambda s: s.get_size()[0])
+    height = property(lambda s: s.get_size()[1])
+
     def operate(self, pos, operation):
         """Internal -
 
-        Common code to calculate the coordinates and get/reset/query a 1/4 character pixel.
+        Common code to calculate the coordinates and get/reset/query a pixel at the correct sub-block resolution.
         Call  :any:`HighRes.set_at`, :any:`HighRes.reset_at` or :any:`HighRes.get_at` instead.
         """
         from terminedia.image import Pixel
@@ -406,9 +460,11 @@ class HighRes:
         p_y = pos[1] // self.block_height
         i_x, i_y = pos[0] % self.block_width, pos[1] % self.block_height
         graphics = True
-        original = self.parent[p_x, p_y]
+        original = self.parent.get_raw((p_x, p_y))
         if isinstance(original, Pixel):
             original = original.value
+        elif isinstance(original, Sequence):
+            original = original[0]
         if original is TRANSPARENT or original not in self.block_class:
             graphics = False
             original = EMPTY
@@ -482,3 +538,94 @@ class HighRes:
         Context's direction is respected when printing
         """
         self.parent.print_at(self.at_parent(pos), text)
+
+
+class Square(HighResBase):
+    """Reimplements the sub-character block plotting to allow seamless full color in 1x2 resolution"""
+
+    def __init__(self, parent, block_class=HalfChars, block_width=1, block_height=2):
+        super().__init__(parent, block_class, block_width, block_height)
+        self.PixelCls = parent.PixelCls
+
+    def set_at(self, pos):
+        """Sets pixel at given coordinate
+
+        Args:
+          - pos (2-sequence): pixel coordinate
+
+        To be used as a callback to ``.draw.set`` - but there are no drawbacks
+        in being called directly.
+        """
+        is_graphics, gross_pos, new_block = self.operate(pos, self.block_class.set)
+        current = self.parent.PixelCls(*self.parent.get_raw(gross_pos))
+
+        if new_block == HalfChars.FULL_BLOCK:
+            if self.context.color == current.foreground:
+                new_pixel = self.PixelCls(HalfChars.FULL_BLOCK, self.context.color, self.context.background, self.context.effects)
+            elif current.foreground is DEFAULT_FG:
+                new_block = HalfChars.UPPER_HALF_BLOCK if pos[1] % 2 == 1 else HalfChars.LOWER_HALF_BLOCK
+                new_pixel = self.PixelCls(new_block, current.foreground, self.context.color, self.context.effects)
+            else:
+                new_block = HalfChars.UPPER_HALF_BLOCK if pos[1] % 2 == 0 else HalfChars.LOWER_HALF_BLOCK
+                new_pixel = self.PixelCls(new_block, self.context.color, current.foreground, self.context.effects)
+        elif current.value == HalfChars.EMPTY or not is_graphics:
+            new_pixel = self.PixelCls(new_block, self.context.color, current.background, self.context.effects)
+        else:  # implies  current.value == new_block and current.value in (HalfChars.UPPER_HALF_BLOCK, HalfChars.LOWER_HALF_BLOCK):
+            new_pixel = current._replace(foreground=self.context.color, effects=self.context.effects)
+
+        self.parent[gross_pos] = new_pixel
+
+    def reset_at(self, pos):
+        """Resets pixel at given coordinate
+
+        Args:
+          - pos (2-sequence): pixel coordinate
+
+        To be used as a callback to ``.draw.reset`` - but there are no drawbacks
+        in being called directly.
+        """
+        is_graphics, gross_pos, new_block = self.operate(pos, self.block_class.reset)
+        current = self.parent.PixelCls(*self.parent.get_raw(gross_pos))
+
+        if new_block == HalfChars.EMPTY:
+            if self.context.background == current.background:
+                new_pixel = self.PixelCls(HalfChars.EMPTY, self.context.color, self.context.background, self.context.effects)
+            elif current.background is DEFAULT_BG:
+                new_block = HalfChars.UPPER_HALF_BLOCK if pos[1] % 2 == 0 else HalfChars.LOWER_HALF_BLOCK
+                new_pixel = self.PixelCls(new_block, self.context.background, current.background, self.context.effects)
+            else:
+                new_block = HalfChars.UPPER_HALF_BLOCK if pos[1] % 2 == 1 else HalfChars.LOWER_HALF_BLOCK
+                new_pixel = self.PixelCls(new_block, current.background, self.context.background, self.context.effects)
+        elif current.value == HalfChars.FULL_BLOCK:
+            new_pixel = self.PixelCls(new_block, current.foreground, self.context.background, self.context.effects)
+        else:  # implies  current.value == new_block and current.value in (HalfChars.UPPER_HALF_BLOCK, HalfChars.LOWER_HALF_BLOCK):
+            new_pixel = current._replace(background=self.context.background, effects=self.context.effects)
+
+        self.parent[gross_pos] = new_pixel
+
+    def get_at(self, pos):
+        """Queries pixel at given coordinate
+
+        Args:
+          - pos (2-sequence): pixel coordinate
+
+        Returns:
+           - True: pixel is set
+           - False: pixel is not set
+           - None: Character on Screen at given coordinates is not a block character of the class
+                used as pixel-characters for this instance.
+        """
+
+        is_graphics, gross_pos, is_set = self.operate(pos, self.block_class.get_at)
+        current = self.parent.get_raw(gross_pos)
+        # foreground ...else... background
+        return current[1] if is_set else current[2]
+
+
+def HighRes(parent, block_class=BlockChars, block_width=2, block_height=2):
+    """Factory method - uses specialized class if Square resolution for a shape supporting FG & BG is requested"""
+
+    cap = parent.PixelCls.capabilities if hasattr(parent, "PixelCls") else None
+    full_color = cap and cap.has_foreground and cap.has_background
+    cls = Square if full_color and block_width == 1 and block_height == 2 else HighResBase
+    return cls(parent, block_class, block_width, block_height)
