@@ -2,18 +2,22 @@ import heapq
 import logging
 import math
 import sys
+import threading
 from abc import ABC, abstractmethod
-from collections import namedtuple
+from collections import namedtuple, ChainMap, deque
 from collections.abc import Sequence
+from functools import wraps
 from inspect import signature
+from itertools import chain
 from io import StringIO
 from pathlib import Path
 from weakref import ref, ReferenceType
 
 from terminedia.contexts import Context
 from terminedia.sprites import SpriteContainer
-from terminedia.subpixels import BrailleChars, HalfChars
-from terminedia.utils import Color, Rect, V2, LazyBindProperty, char_width, get_current_tick
+from terminedia.subpixels import BrailleChars, HalfChars, SextantChars
+from terminedia.unicode import char_width
+from terminedia.utils import Color, Rect, V2, LazyBindProperty, get_current_tick, size_in_blocks
 from terminedia.unicode_transforms import translate_chars
 from terminedia.values import (
     DEFAULT_FG,
@@ -194,6 +198,10 @@ class ShapeApiMixin:
     def braille(self):
         return self._get_highres(block_class=BrailleChars, block_height=4)
 
+    @LazyBindProperty
+    def sextant(self):
+        return self._get_highres(block_class=SextantChars, block_height=3)
+
     @LazyBindProperty
     def sprites(self):
         self.has_sprites = True
@@ -220,6 +228,7 @@ class ShapeApiMixin:
                 self, pos, pixel if pixel else self.context.char
             ),
             reset_fn=lambda pos: type(self).__setitem__(self, pos, EMPTY),
+            get_fn=lambda pos: type(self).get_raw(self, pos),
             size_fn=self.get_size,
             context=self.context,
         )
@@ -230,9 +239,9 @@ class ShapeApiMixin:
         return HighRes(self, **kw)
 
     def _get_text(self):
-        from terminedia.text import Text
+        from terminedia.text import TextPlane
 
-        return Text(self)
+        return TextPlane(self)
 
     def clear(self, transparent=False):
         """Clear the shape with empty spaces.
@@ -256,6 +265,13 @@ class ShapeApiMixin:
             self.draw.fill()
         self.dirty_set()
 
+    def spaces_to_transparency(self):
+        from terminedia.transformers.library import AddAlpha
+        from terminedia.transformers import TransformersContainer
+        ct = TransformersContainer((AddAlpha,))
+        with self.context(force_transparent_ink=True):
+            ct.bake(self)
+
 
 #####################
 #
@@ -351,6 +367,8 @@ class ShapeDirtyMixin:
         self.dirty_registry.reset()  # clear_left(tick)
 
         self.dirty_save_current_sprite_rects(tick)
+        for sprite in self.sprites:
+            sprite.shape.dirty_clear()
 
     def dirty_save_current_sprite_rects(self, tick):
         self.dirty_sprite_rects_saved_at = tick
@@ -376,7 +394,7 @@ class ShapeDirtyMixin:
         # If there is any time-dependant image change, there is no way
         # to predict what changes from one frame to the next - just mark
         # all shape as dirty.
-        if any("tick" in transformer.signatures for transformer in self.context.transformers):
+        if any("tick" in sig for transformer in self.context.transformers for sig in transformer.signatures.values()):
             self.dirty_set()
             return
 
@@ -386,8 +404,15 @@ class ShapeDirtyMixin:
                 if not sprite.active:
                     continue
 
-                for rect in sprite.dirty_rects:
-                    self.dirty_registry.push((tick, sprite.owner_coords(rect), sprite.shape))
+                if any("tick" in sig for transformer in sprite.transformers for sig in transformer.signatures.values()):
+                    self.dirty_registry.push((tick, sprite.rect, sprite.shape))
+                else:
+                    for rect in sprite.dirty_rects:
+                        self.dirty_registry.push((tick, sprite.owner_coords(rect), sprite.shape))
+            if self.sprites.killed_sprites:
+                for rect in self.sprites.killed_sprites:
+                    self.dirty_registry.push((tick, rect, None))
+                self.sprites.killed_sprites.clear()
 
         # mark dirty pixels
 
@@ -431,6 +456,9 @@ class Shape(ABC, ShapeApiMixin, ShapeDirtyMixin):
     """
 
     PixelCls = pixel_factory(bool)
+    _default_bg = False
+
+    isroot = False
 
     @classmethod
     def new(cls, size, **kwargs):
@@ -608,14 +636,14 @@ class Shape(ABC, ShapeApiMixin, ShapeDirtyMixin):
         original_output = output
         if isinstance(output, (str, Path)):
             output = open(
-                output, "wt"
-            )  # FIXME: for some backends a binary file will be needed.
+                output, "w" + ("t" if backend in ("ANSI", "HTML") else "b")
+            )
         if not original_output:
             output = StringIO()
 
         if backend == "ANSI":
-            return self._render_using_screen(output, backend)
-        if backend == "HTML":
+            self._render_using_screen(output, backend)
+        elif backend == "HTML":
             # FIXME: this somewhat violates some "this module should not know about
             # specific backend stuff (HTML template)."
             # However, the rendering machinnery only knows
@@ -628,6 +656,10 @@ class Shape(ABC, ShapeApiMixin, ShapeDirtyMixin):
             output.write(preamble)
             self._render_using_screen(output, backend)
             output.write(post_amble)
+        elif backend == "SNAPSHOT":
+            import pickle
+            tmp = ShalowShapeRepr(self)
+            pickle.dump(tmp, output)
         else:
             raise ValueError(f"Output type {backend!r} not implemented")
         if not original_output:
@@ -640,7 +672,7 @@ class Shape(ABC, ShapeApiMixin, ShapeDirtyMixin):
             file = StringIO()
         else:
             file = output
-        sc = Screen(size=V2(self.width, self.height), backend=backend)
+        sc = Screen(size=V2(self.width, self.height), backend=backend, interactive=False)
         if backend=="ANSI":
             # generate a relocatable image
             sc.commands.__class__.last_pos = V2(0, 0)
@@ -653,6 +685,7 @@ class Shape(ABC, ShapeApiMixin, ShapeDirtyMixin):
         sc.commands.stop_journal()
         # Renders all graphic ops as ANSI sequences + unicode into file:
         sc.commands.replay(output)
+        output.write("\x1b[0m")  # reset all ansi attributes
 
     def __repr__(self):
         cap = self.PixelCls.capabilities
@@ -674,6 +707,35 @@ class Shape(ABC, ShapeApiMixin, ShapeDirtyMixin):
         )
         return rep
 
+    def _resize_data_one(self, new_size, data, fill_value):
+        old_size = V2(self.width, self.height)
+        lines = [data[i : i + old_size.x] for i in range(0, len(data), old_size.x)]
+        new_data = []
+        diff = new_size.x - old_size.x
+        for y, line in zip(range(new_size.y), lines):
+
+            if diff > 0:
+                line.extend([fill_value] * diff)
+            elif diff < 0:
+                line[diff:] = []
+            new_data.extend(line)
+
+        y_diff = new_size.y - old_size.y
+        if y_diff > 0:
+            new_data.extend([fill_value] * new_size.x * y_diff)
+
+        return new_data
+
+    def _resize_data(self, new_size):
+        self.data = self._resize_data_one(new_size, self.data, fill_value=getattr(self.context, "background_char", self.__class__._default_bg))
+
+    def resize(self, new_size):
+        new_size = V2(new_size)
+        self._resize_data(V2(new_size))
+        self.width, self.height = new_size
+        if hasattr(self, "rect"):
+            self.rect = Rect(new_size)
+        self.dirty_set()
 
 
 # "Virtualsubclassing" - 2 days after I wrote there were no
@@ -683,6 +745,8 @@ class Shape(ABC, ShapeApiMixin, ShapeDirtyMixin):
 class ShapeView(ShapeApiMixin):
     __slots__ = ("roi", "original", "_draw", "_high", "_text")
 
+    isroot = False
+
     def __init__(self, original, roi):
         self.original = original
         self.roi = Rect(roi)
@@ -725,6 +789,7 @@ class ShapeView(ShapeApiMixin):
             "draw",
             "high",
             "text",
+            "isroot",
             "_draw",
             "_high",
             "_text",
@@ -923,12 +988,17 @@ class ImageShape(ValueShape):
             color = tuple(color)
         self.data.putpixel(pos, color)
 
+    def resize(self, new_size):
+        self.data = self.data.resize(new_size)
+        self.width, self.height = self.data.width, self.data.height
+
     def clear(self, transparent=False):
         img = self.data
         # FIXME: might need to check and upgrade the image to RGBA first
         color = tuple(self.context.background) if not transparent else (0, 0, 0, 0)
         img.paste(tuple(self.context.background), [0,0, img.size[0], img.size[1]])
 
+
 class PalettedShape(Shape):
     """'Shape' class intended to represent images, using a color-map to map characters to block colors.
 
@@ -946,6 +1016,7 @@ class PalettedShape(Shape):
     effects = False  # FUTURE: support for bold, blink, underline...
 
     PixelCls = pixel_factory(bool, has_foreground=True)
+    _default_bg = False
 
     def __init__(self, data, color_map=None):
         if color_map is None:
@@ -1010,10 +1081,170 @@ class PalettedShape(Shape):
         pos = V2(pos)
         self.dirty_mark_pixel(pos)
         type_ = self.PixelCls.capabilities.value_type
+        self.raw_setitem(pos, type_(value))
+
+    def _raw_setitem(self, pos, value):
         self.data[pos[1] * self.width + pos[0]] = type_(value)
 
 
-class FullShape(Shape):
+
+class PixelDict(ChainMap):
+    """Cached ChainMap"""
+    def __init__(self, maps=None):
+        # Chainmap uses a plain list, still, maps are pilld top-most on the left-side.
+        # go figure!
+        super().__init__(maps or {})
+        self.maps = deque(self.maps)
+        self.cache = {}
+        self._sentinel = object()
+
+    def __setitem__(self, key, value):
+        self.cache[key] = value
+        super().__setitem__(key, value)
+
+    def __getitem__(self, key):
+        value = self.cache.get(key, self._sentinel)
+        if value is self._sentinel:
+            value = super().__getitem__(key)
+            self.cache[key] = value
+        return value
+
+    def clear(self):
+        self.maps = [{}]
+        self.cache.clear()
+
+    def pop(self):
+        self.cache.clear()
+        return self.maps.popleft()
+
+    def push(self, map=None):
+        if map:
+            self.cache.clear()
+        self.maps.appendleft(map or {})
+
+
+class _UNDO_START_MARK:
+    pass # sentinel
+
+class _UNDO_IN_PROGRESS_MARK:
+    pass # sentinel
+
+
+class RasterUndo:
+    """Controls and offer the API for raster undo-capability
+
+    """
+    # FIXME: possibly we will need extracontext.context instead of threading.local
+    # (maybe even contextvars.ContextVar could work)
+    _undo_registry = threading.local()
+
+    def __init__(self, *args, undo_active=False, max_undo_steps=100, **kw):
+        # self.__lock = threading.Lock()
+        self.max_undo_steps = max_undo_steps
+        self.redo_data = []
+        self.undo_active = undo_active
+        super().__init__(*args, **kw)
+
+    def __undo_exit(self): #, ext_type, exc_value, tb):
+        with self.__lock:
+            self.__undo_deph -= 1
+            # we don't pop or merge undo-groups: that is up to the app do by calling other functions;
+
+    def undo(self, n=1):
+        for i in range(n):
+            if len(self.data.maps) <= 1:
+                break
+            self.redo_data.append(self.data.pop())
+        if isinstance(self, ShapeDirtyMixin):
+            self.dirty_set()
+
+    def redo(self, n=1):
+        for i in range(n):
+            if not self.redo_data:
+                break
+            self.data.maps.push(self.redo_data.pop())
+        if isinstance(self, ShapeDirtyMixin):
+            self.dirty_set()
+
+    def undo_clear(self, n=1):
+        """Merge all pixel data into base, and clear undo history"""
+        base = self.data.maps[-1]
+        for step in self.data.maps[-2::-1]:
+            base.update(step)
+        self.data.clear()
+        self.data.maps[0] = base
+
+    @classmethod
+    def undoable(cls, func):
+        """Decorator - apply on functions and methods that will use Raster functions so that
+        they automatically start an undo_group.
+
+        '@FullShape.undoable'
+        """
+        return cls._inner_undoable(func, _inner_func=False)
+
+    @classmethod
+    def _inner_undoable(cls, func, *, _inner_func=True):
+        """Undo marker, but for 'final' methods inside the undoable-shape class itself:
+        this will finally know the actual instance where undoing is expected,
+        and will interact with tokens set-up in the outher function/methods
+        (decorated with 'undoable') to actually create the undo-group dictionary to
+        be stacked.
+        """
+        @wraps(func)
+        def undo_wrapper(*args, **kwargs):
+            class_markers = getattr(cls._undo_registry, "class_markers", None)
+            if class_markers is None:
+                class_markers = cls._undo_registry.class_markers = {}
+            outter_level = False
+            if "state" not in class_markers:
+                # FIXME: the 'key' here would be a unique 'chain-call-lineage'
+                # starting on the outermost undoable function, and that
+                # would not be mixed across threads/asyncio_tasks
+                # meanwhile, the fixed key "state" will do
+                class_markers["state"] = _UNDO_START_MARK
+                outter_level = True
+            if _inner_func and class_markers["state"] is _UNDO_START_MARK:
+                self = args[0]
+                # new undo group
+                if self.undo_active:
+                    self.data.push()
+                    class_markers["state"] = _UNDO_IN_PROGRESS_MARK
+                    self.verify_and_merge_max_undo_groups()
+                # FIXME: maybe think of a non-linear redo strategy?
+                self.redo_data.clear()
+            try:
+                result = func(*args, **kwargs)
+            finally:
+                if outter_level:
+                    del class_markers["state"]
+            return result
+
+        return undo_wrapper
+
+    def undo_group_start(self):
+        if self.undo_active:
+            class_markers = getattr(self.__class__._undo_registry, "class_markers", None)
+            class_markers["state"] = _UNDO_START_MARK
+            self.verify_and_merge_max_undo_groups()
+
+    def undo_group_end(self):
+        class_markers = getattr(self.__class__._undo_registry, "class_markers", None)
+        if class_markers.get("state") is not None:
+            del class_markers["state"]
+
+    def verify_and_merge_max_undo_groups(self):
+        while len(self.data.maps) > self.max_undo_steps + 1:
+            # merge the third-to-last and second-to-last groups:
+            if self.max_undo_steps >= 2:
+                self.data.maps[-2].update(self.data.maps[-3])
+                del self.data.maps[-3]
+            else:
+                self.data.maps[-1].update(self.data.maps[-2])
+                del self.data.maps[-2]
+
+
+class FullShape(RasterUndo, Shape):
     """Shape class carrying all possible data plus kitchen sink
 
     Args:
@@ -1031,36 +1262,56 @@ class FullShape(Shape):
         has_effects=True,
         translate_dots=False,
     )
+    _default_bg = EMPTY
 
     @staticmethod
-    def _data_func(size):
+    def _data_func(size, context=None):
+        if context is None:
+            import terminedia
+            context = terminedia.context
         return [
             [EMPTY * size.x] * size.y,
-            [DEFAULT_FG] * size.x * size.y,
-            [DEFAULT_BG] * size.x * size.y,
-            [Effects.none] * size.x * size.y,
+            [context.foreground] * size.x * size.y,
+            [context.background] * size.x * size.y,
+            [context.effects] * size.x * size.y,
         ]
 
-    def __init__(self, data):
+    def __init__(self, data, **kw):
         self.width = w = len(data[0][0])
         self.height = h = len(data[0])
-        self.value_data, self.fg_data, self.bg_data, self.eff_data = (
-            self.load_data(plane, (w, h)) for plane in data
-        )
+        self.rect = Rect((w,h))
+        self.load_data(data, (w,h))
+        #self.value_data, self.fg_data, self.bg_data, self.eff_data = (
+            #self.load_data(plane, (w, h)) for plane in data
+        #)
         # self.data is created as a side-effect in load_data
-        del self.data
-        super().__init__()
+        #del self.data
+        super().__init__(**kw)
+
+    def load_data(self, data_planes, size):
+        original_undo = getattr(self, "undo_active", False)
+        self.undo_active = False
+        w, h = size
+        self.data = PixelDict()
+        data_planes[0] = chain(*data_planes[0])
+        iter_data = zip(*data_planes)
+        for y in range(h):
+            for x in range(w):
+                self._raw_setitem((x, y), next(iter_data))
+        self.undo_active = original_undo
 
     def get_raw(self, pos):
-        offset = self.get_data_offset(pos)
-        if offset is None:
-            return (EMPTY, self.context.color, self.context.background, Effects.none)
-        return (
-            self.value_data[offset],
-            self.fg_data[offset],
-            self.bg_data[offset],
-            self.eff_data[offset],
-        )
+        if isinstance(pos, list):
+            pos = V2(pos)
+        value = self.data.get(pos, None) if pos in self.rect else None
+
+        if value is None:
+            value = [EMPTY, self.context.color, self.context.background, self.context.effects]
+            self.data[pos] = value
+        if self.undo_active:
+            # mutations have to be written back, so they are placed in to the chainned undo_group
+            value = value[:]
+        return value
 
     def __getitem__(self, pos):
         """Values for each pixel are: character, fg_color, bg_color, effects.
@@ -1077,6 +1328,7 @@ class FullShape(Shape):
             pixel = self.sprites.get_at(pos, pixel)
         return pixel
 
+    @RasterUndo._inner_undoable
     def __setitem__(self, pos, value):
         """
         Values set for each pixel are: character - only spaces (0x20) or "non-spaces" are
@@ -1087,8 +1339,8 @@ class FullShape(Shape):
 
         force_transparent_ink = getattr(self.context, "force_transparent_ink", False)
 
-        offset = self.get_data_offset(pos)
-        if offset is None:
+        #offset = self.get_data_offset(pos)
+        if pos not in self.rect:
             return
         if isinstance(value, Pixel):
             value = value.get_values(self.context, self.PixelCls.capabilities)
@@ -1102,16 +1354,26 @@ class FullShape(Shape):
                 self.context.effects,
             ][len(value) - 1 :]
 
+        #####################
+        # Apply pre-transformers: backed in transformations specified in context.
+        #####################
+
+        if self.context.pretransformers:
+            value = self.context.pretransformers.process(self, pos, self.PixelCls(*value))
+
         ############
-        # Check final width (have to apply transformation effect)
+        # Check final width (after have to apply transformation effect)
         ###########
-        effects = value[3] if (value[3] != TRANSPARENT or force_transparent_ink) else self.eff_data[offset]
+        offset2 = None
+
+        effects = value[3] if (value[3] != TRANSPARENT or force_transparent_ink) else self.get_raw(pos)[3]
         transform_effects = (effects & UNICODE_EFFECTS) if effects != TRANSPARENT else Effects.none
+        # FIXME: check for unicode combining gliphs
         final_char = value[0]
-        if isinstance(final_char, bool):
+        if isinstance(final_char, (bool, int)):
             final_char = self.context.char if final_char else EMPTY
         if final_char == CONTINUATION:
-            if self.value_data[offset] == CONTINUATION:
+            if self.get_raw(pos)[0] == CONTINUATION:
                 # we are likely being blitted from a source with matching parameters.
                 # attributes are already set in this cell from
                 # previous character setting
@@ -1122,45 +1384,110 @@ class FullShape(Shape):
                 final_char = translate_chars(value[0], transform_effects)
             double_width = char_width(final_char) == 2
             if double_width:
-                if pos[0] == self.width - 1:  # Right shape edge
-                    width = 1
-                    double_width = False
+                if not getattr(self.context, "text_rendering_styled", None) == 1:
+                    if pos[0] == self.width - 1:  # Right shape edge
+                        double_width = False
+                    else:
+                        offset2 = pos[0] + 1
                 else:
-                    offset2 = offset + 1
+                    # a character sequence of styled-text is being rendered.
+                    if pos[0] == 0:
+                        # FIXME: if a double-width char hits the edge in RTL
+                        # printing, this have to be handled in higher level
+                        pass
+                    if self.context.direction == Directions.LEFT:
+                        # EXPERIMENTAL: change actual target in this
+                        # situation (rendering_text and going left)
+                        # and leave a CONTINUATION marker on the target position.
+                        offset2 = pos[0]
+                        pos = list(pos)
+                        pos[0] -= 1
+                    else:
+                        if pos[0] == self.width - 1:  # Right shape edge
+                            double_width = False
+                        offset2 = pos[0] + 1
         else:
             double_width = False
-        # /check width
-        for component, plane in zip(
-            value, (self.value_data, self.fg_data, self.bg_data, self.eff_data)
-        ):
-            if component is not TRANSPARENT or force_transparent_ink:
-                plane[offset] = component
-            if double_width:
-                plane[offset2] = (
-                    component if plane is not self.value_data else CONTINUATION
-                )
-        # set information so higher level users can partake char width (text, blit)
         self.context.shape_lastchar_was_double = double_width
+        self._raw_setitem(pos, value, force_transparent_ink, double_width, offset2)
+
+    def _raw_setitem(self, pos, value, force_transparent_ink=False, double_width=False, offset2=None):
+        pixel = self.get_raw(pos)
+        if offset2:
+            pixel2 = self.get_raw((offset2, pos[1]))
+        for i, component in enumerate(value):
+            # the idea is that "TRANSPARENT" won't affect the corresponding component.
+            # but "force_transparent_ink" can set the value of the component itself to
+            # be the "transparent" special marker
+            if component is not TRANSPARENT or force_transparent_ink:
+                pixel[i] = component
+                if double_width:
+                    pixel2[i] = component if i != 0 else CONTINUATION
+        if self.undo_active:
+            self.data[pos] = pixel
+
+            if offset2:
+                self.data[offset2, pos[1]] = pixel2
+
+    def _resize_data(self, new_size):
+        return
 
     @classmethod
-    def promote(cls, other_shape):
+    def promote(cls, other_shape, resolution=None):
         """Makes a FullShape copy of the other shape
 
         This allows the new shape to be used with Transformers,
         Sprites, and other elements that might not be supported
         in the other shape classes
         """
-        new_shape = cls.new(other_shape.size)
-        new_shape.draw.blit((0,0), other_shape)
+
+        if not resolution:
+            new_shape = cls.new(other_shape.size)
+            draw = new_shape.draw
+        else:
+            size = size_in_blocks(other_shape.size, resolution)
+            new_shape = cls.new(size)
+            draw = getattr(new_shape, resolution).draw
+
+        draw.blit((0,0), other_shape)
+
         return new_shape
 
+class ShalowShapeRepr:
+    def __init__(self, original: Shape):
+        """The final visible pixel data of a shape as a simple pickleable object
 
-def shape(data, color_map=None, **kwargs):
+        This can be pickled into disk, and later restored into a new shape.
+
+        Warning: This class applies the visible effects and otherwise ignore text-plane information,
+        marks, sprites and any Transformers. A simple Python object is written to disk,
+        which can be retrieved.
+        """
+        self.size = original.size
+        self.data = []
+        self.cls = original.__class__
+        for pos in Rect(self.size).iter_cells():
+            self.data.append(tuple(original[pos]))
+
+    def restore(self):
+        """Recreates a Shape of the original class with the flattened data"""
+        # FIXME: Palleted shapes do not save Palette information
+        shape = self.cls.new(self.size)
+        for pos, value in zip(Rect(self.size).iter_cells(), self.data):
+            shape[pos] = value
+        return shape
+
+
+
+def shape(data, color_map=None, promote=False, resolution=None, **kwargs):
     """Factory for shape objects
 
     Args:
       - data (Filepath to image, open file, image data as text or list of strings)
       - color_map (optional mapping): color map to be used for the image - mapping characters to RGB colors.
+      - promote (boolean): Whether to force resulting shape to a FullShape (defaults to False)
+      - resolution (str): If promote is True, resolution namespace to use on blitting to
+            FullShape ("square", "high", "square")
       - **kwargs: parameters passed transparently to the selected shape class
 
     Based on inferences on the data attribute, selects
@@ -1186,8 +1513,15 @@ def shape(data, color_map=None, **kwargs):
             name = Path(getattr(data, "name", "stream"))
         else:
             name = Path(data)
-        if not PILImage or name.suffix.strip(".").lower() in "pnm ppm pgm":
+        suffix = name.suffix.strip(".").lower()
+        if suffix in "pnm ppm pgm".split():
             cls = PGMShape
+        elif suffix == "snapshot": # FIXME: find a better way to detect a pickle-file
+            import pickle
+            obj = pickle.load(open(data, "rb"))
+            # FIXME:  the future full shapes may be pickled, and no need to call  "restore" method
+            return obj.restore()
+
         else:
             cls = ImageShape
     elif PILImage and isinstance(data, PILImage.Image):
@@ -1200,4 +1534,7 @@ def shape(data, color_map=None, **kwargs):
         return FullShape.new(data, **kwargs)
     else:
         raise NotImplementedError("Could not pick a Shape class for given arguments!")
-    return cls(data, color_map, **kwargs)
+    result = cls(data, color_map, **kwargs)
+    if promote:
+        result = FullShape.promote(result, resolution=resolution)
+    return result
